local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")
local const = require('const')
local request_method = ngx.var.request_method
local ldbus = require "ldbus"

utils.check_login()
utils.check_permission("DeviceManage", "GatewayManage")

function generate_random_num()
    math.randomseed(tostring(os.time()):reverse():sub(1, 7))
    return math.random(1,100)
end

function get_pk_dn_from_interface(interface, prefix)
    local s = utils.split_str(interface, "_")
    local pk = string.sub(s[1], string.len(prefix)+1, string.len(s[1]))
    local dn = s[2]

    return pk, dn
end

function receive (pk, dn)
    local prefix = "iot.device.id"
    local postfix = "_"
    local cur_worker_pid = ngx.worker.pid()
    local wkn = "linkedge.webconsole.a"..cur_worker_pid..generate_random_num()
	local device_local_id = prefix..pk..postfix..dn
    
    utils.log_error ( "Listening for signals, pid: " ..cur_worker_pid.." -->:  "..wkn)

	local conn = assert ( ldbus.connection.open ( const.linkedge.bus_addr ) )

    if (ldbus.bus.register(conn, "register failed"))
    then
        utils.log_error("register succeed.")
    else
        utils.log_error("failed to register dbus")
    end
    assert ( ldbus.bus.request_name ( conn , wkn , { replace_existing = true } ) )
    local msg = assert ( ldbus.message.new_method_call ( const.subscribe.wkn,  const.subscribe.obj, const.subscribe.wkn , const.subscribe.interface_sub) , "Message Null" )
    local iter = ldbus.message.iter.new ( )
    msg:iter_init_append ( iter )
    assert ( iter:append_basic ( string.format(const.subscribe.fmt_sub, pk, dn, wkn)) , "Out of Memory" )
    local reply = assert ( conn:send_with_reply_and_block ( msg ) )
    
    assert ( ldbus.bus.add_match ( conn , "type='signal',interface='"..device_local_id.."'" ) )
	conn:flush ( )

	utils.log_error ( "Match rule sent for "..device_local_id)

	while conn:read_write ( 1*1000 ) do
		local msg = conn:pop_message ( )
        local res = '{"code": 204, "message": "heartbeats", "data": null}'
        if msg and msg:get_type ( ) == "signal" and msg:get_interface( ) ~= "org.freedesktop.DBus" then
			local iter = ldbus.message.iter.new ( )
			assert ( msg:iter_init ( iter ) , "Message has no parameters" )
			assert ( iter:get_arg_type ( ) == ldbus.types.string , "Argument is not a string" )
			local sigvalue = iter:get_basic ()
            local msg_type = msg:get_member()
            local pk, dn = get_pk_dn_from_interface(msg:get_interface(), prefix) 
            res = '{"code": 200, "message": "success", "data":{"productKey":"'..pk..'","deviceName":"'..dn..'","type":"'..msg_type..'","params":'..sigvalue .."}}"
        end
        
       	ngx.header['Transfer-Encoding'] = 'chunked'
        ok, err = ngx.say(res)
        ngx.flush(true)

        if ok
            then
                utils.log_error("send status: " ..ok)
            else
                utils.log_error("remote client offline now")
                ngx.exit(ngx.HTTP_OK)
        end
	end
    utils.log_error("clear dbus connection...")
end

if ("GET" == request_method) then
   local args = ngx.req.get_uri_args()
   
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end
   
    pk = args["ProductKey"]
    dn = args["DeviceName"]
    
    for k, v in pairs(args) do
        utils.log_error("key: "..k.." val: "..v)
    end

    utils.log_error("i am stupid pk: "..pk.." dn "..dn)

    utils.check_pk(pk)
    utils.check_dn(dn)

    receive (pk, dn)
end
